/* global jest describe beforeEach it expect */

'use strict';

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _reactAddons = require('react/addons');

var _reactAddons2 = _interopRequireDefault(_reactAddons);

jest.dontMock('../components/Separator');

var TestUtils = _reactAddons2['default'].addons.TestUtils;

// Babel would move an import in front of the jest.dontMock. That's why require
// is used instead of import.
var Separator = require('../components/Separator');

describe('Separator', function () {
  it('should come with a set of default styles', function () {
    var separator = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(
      Separator,
      null,
      'Europe'
    ));

    var div = TestUtils.findRenderedDOMComponentWithTag(separator, 'div');
    expect(div.props.style.color).toBe('#666');
  });

  it('should be able to overwrite the default styles', function () {
    var separator = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(
      Separator,
      { style: { color: '#F00' } },
      'Please select a city'
    ));

    var div = TestUtils.findRenderedDOMComponentWithTag(separator, 'div');
    expect(div.props.style.color).toBe('#F00');
  });

  it('should be able to provide custom properties', function () {
    var separator = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(
      Separator,
      { 'data-custom': 'example' },
      'Please select a city'
    ));

    var div = TestUtils.findRenderedDOMComponentWithTag(separator, 'div');
    expect(div.props['data-custom']).toBe('example');
  });
});